require "logger"
require "json"
require "net/http"
require "uri"

class SlackLogger < Logger
  VERSION = [
    VERSION_MAJOR = 1,
    VERSION_MINOR = 0,
    VERSION_TINY = 0,
    VERSION_PRE = nil
  ].compact.join(".")

  class Error < RuntimeError; end
  class DatetimeFormatNotSupported < Error; end
  class FormatterNotSupported < Error; end
  class UndefinedSlackHookUrl < Error; end
  class UndefinedSlackChannel < Error; end

  attr_accessor :slack_channel
  attr_accessor :slack_hook_url
  attr_accessor :slack_icon_emoji
  attr_accessor :slack_icon_url
  attr_accessor :slack_username

  def initialize(options = {})
    @slack_hook_url = options.fetch(:slack_hook_url) { nil }
    @slack_channel = options.fetch(:slack_channel) { nil }
    @slack_username = options.fetch(:slack_username) { "Logger" }
    @slack_icon_url = options.fetch(:slack_icon_url) { nil }
    @slack_icon_emoji = options.fetch(:slack_icon_emoji) { nil }
    @level = DEBUG
    @logdev = nil
  end

  def formatter=(formatter)
    raise(FormatterNotSupported)
  end

  def formatter
    raise(FormatterNotSupported)
  end

  def datetime_format=(datetime_format)
    raise(DatetimeFormatNotSupported)
  end

  def datetime_format
    raise(DatetimeFormatNotSupported)
  end

  def <<(message)
    error(message)
  end

  def add(severity, message = nil, progname = nil, &block)
    raise(UndefinedSlackHookUrl) unless @slack_hook_url
    raise(UndefinedSlackChannel) unless @slack_channel

    severity ||= FATAL
    message ||= block_given? ? yield : progname

    return true if severity < @level

    uri = URI.parse(slack_hook_url)
    payload = slack_payload(severity, message).to_json

    Net::HTTP.post_form(uri, payload: payload)
  end

  private

  def slack_payload(severity, message)
    {
      channel: slack_channel,
      username: slack_username,
      icon_url: slack_icon_url,
      icon_emoji: slack_icon_emoji,
      text: "",
      attachments: [{
        color: slack_color(severity),
        text: message
      }]
    }
  end

  def slack_color(severity)
    ["#e3e4e6", "#55acee", "warning", "danger"][severity] || "danger"
  end
end
